// IndentedTrace.h: interface for the CIndentedTrace class.
//
// Better tracing class and macros
// 
// Copyright (C) 2000 Mark Messer - Released to the public domain
// This software is provided "as is" without warranty of any kind. 
// 
// Revision History
// 11/28/00	MM	Initial release
// 
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_INDENTEDTRACE_H__8036B778_7381_11D1_84E6_0080C8393E4D__INCLUDED_)
#define AFX_INDENTEDTRACE_H__8036B778_7381_11D1_84E6_0080C8393E4D__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

//////////////////////////////////////////////////////////////////////
class CIndentedTrace  
{
#ifdef _DEBUG

private:
	static int ms_iTraceDepth;
	static int ms_iSuppressDepth;
	static int ms_iSerialNumber;

	static bool ms_bEnableSerialNumber;
	static bool ms_bEnableThreadID;

	int m_nLocalTraceDepth;
	int m_nLocalSerialNumber;
	
	enum 
	{ 
		INDENT_SPACES = 3
	};

public:
		// For use by macros like IT_COMMENTn. May be changed at any time by 
		// internal functions. Must be public so macros can use it. 
	CString m_msg;	

public:
	inline int GetSN()		const {return m_nLocalSerialNumber;}

#endif // _DEBUG

private:
	void AppendSN(CString& strText);
	void AppendThdId(CString& strText);

public:
	void Entry(LPCTSTR strClass_Func);
	void Exit();

	static void EnableSerialNumber(bool bEnable);
	static void EnableThreadID(bool bEnable);

private:
	CIndentedTrace();	// Forbid default constructor. Declared, but not def'd.

public:
	CIndentedTrace(LPCTSTR strClass_Func);
	virtual ~CIndentedTrace();

	void Comment(LPCTSTR strComment);
	void SuppressTrace();

	void DumpStr(LPCTSTR str, const UINT NChars, const UINT CharsPerLine = 16);
	void TraceIOErr();
};


//////////////////////////////////////////////////////////////////////
// MACROS
//
// CIndentedTrace is a diagnostic class. It is intended to be like TRACE - 
// it is available for use in debug compiles but disappears during release 
// compiles. This is done by putting macros in source code instead of using 
// CIndentedTrace directly. 

#ifdef _DEBUG		// --- Debug builds ---

// --------------------------------------------------------------------------
// In debug builds, the IT_IT() and IT_EMBEDDED_IT macros create a variable of 
// type CIndentedTrace. It is given the name _IT_vnirrrdaewu because this is 
// an unlikely name for a program to contain. 
// 
// The other macros all use _IT_vnirrrdaewu.

#define IT_IT(str)				CIndentedTrace _IT_vnirrrdaewu(str)
#define IT_EMBEDDED_IT			CIndentedTrace _IT_vnirrrdaewu
#define IT_EMBEDDED_INIT(str)	_IT_vnirrrdaewu(str)
#define IT_ENTRY(str)			_IT_vnirrrdaewu.Entry(str)
#define IT_EXIT					_IT_vnirrrdaewu.Exit()
#define IT_COMMENT(str)			_IT_vnirrrdaewu.Comment(str)
#define IT_SUPPRESS_TRACE		_IT_vnirrrdaewu.SuppressTrace()
#define IT_DUMP_STR(str, n, c)	_IT_vnirrrdaewu.DumpStr((str), (n), (c))
#define IT_DUMP_STR16(str, n)	_IT_vnirrrdaewu.DumpStr((str), (n))
#define IT_TRACE_IO_ERR			_IT_vnirrrdaewu.TraceIOErr()

#define IT_COMMENT1(str, x1)	_IT_vnirrrdaewu.m_msg.Format((str),(x1));		\
								_IT_vnirrrdaewu.Comment(_IT_vnirrrdaewu.m_msg)
#define IT_COMMENT2(str, x1, x2)											\
						_IT_vnirrrdaewu.m_msg.Format((str),(x1),(x2));		\
						_IT_vnirrrdaewu.Comment(_IT_vnirrrdaewu.m_msg)
#define IT_COMMENT3(str, x1, x2, x3)										\
						_IT_vnirrrdaewu.m_msg.Format((str),(x1),(x2),(x3));	\
						_IT_vnirrrdaewu.Comment(_IT_vnirrrdaewu.m_msg)

#define IT_ENABLE_SERIAL_NUM(bEnable)	EnableSerialNumber(bEnable)
#define IT_ENABLE_THREAD_ID(bEnable)	EnableThreadID(bEnable)

// --------------------------------------------------------------------------
#else // _DEBUG	not defined	// --- Release builds ---

// In release builds, the macros are defined as nothing, much like TRACE().

#define IT_IT(str)
#define IT_EMBEDDED_IT
#define IT_ENTRY(str)			
#define IT_EXIT					
#define IT_COMMENT(str)			
#define IT_SUPPRESS_TRACE		
#define IT_DUMP_STR(str, n, c)	
#define IT_DUMP_STR(str, n)		
#define IT_DUMP_STR16(str, n)
#define IT_TRACE_IO_ERR	

#define IT_COMMENT1(str, x1)
#define IT_COMMENT2(str, x1, x2)
#define IT_COMMENT3(str, x1, x2, x3)

#define IT_ENABLE_SERIAL_NUM(bEnable)
#define IT_ENABLE_THREAD_ID(bEnable)

#endif // _DEBUG

#endif // !defined(AFX_INDENTEDTRACE_H__8036B778_7381_11D1_84E6_0080C8393E4D__INCLUDED_)
