// TraceMacroView.cpp : implementation of the CTraceMacroView class
//
// Demo of CIndentedTrace features and macros
// 
// Copyright (C) 2000 Mark Messer - Released to the public domain
// This software is provided "as is" without warranty of any kind. 
// 
// Revision History
// 11/28/00	MM	Initial release
// 
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TraceMacro.h"

#include "TraceMacroDoc.h"
#include "TraceMacroView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTraceMacroView

IMPLEMENT_DYNCREATE(CTraceMacroView, CView)

BEGIN_MESSAGE_MAP(CTraceMacroView, CView)
	//{{AFX_MSG_MAP(CTraceMacroView)
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTraceMacroView construction/destruction

// --------------------------------------------------------------------------
CTraceMacroView::CTraceMacroView()
:
	IT_EMBEDDED_INIT( "CTraceMacroView::CTraceMacroView()" ), 
	m_Importance( 0 )
{
	IT_COMMENT( "CTraceMacroView::CTraceMacroView() - Entering constructor body" );
	
	m_pVeryImportance = new CVeryImportance();
	ASSERT( m_pVeryImportance != NULL );

	IT_EXIT;
}


// --------------------------------------------------------------------------
CTraceMacroView::~CTraceMacroView()
{
	IT_ENTRY( "CTraceMacroView::~CTraceMacroView()" );

	delete m_pVeryImportance;

	IT_COMMENT( "CTraceMacroView::CTraceMacroView() - Leaving destructor body" );
}


// --------------------------------------------------------------------------
BOOL CTraceMacroView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CTraceMacroView drawing

// --------------------------------------------------------------------------
void CTraceMacroView::OnDraw(CDC* pDC)
{
	CTraceMacroDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC->TextOut(20,  20, CString("Before running this program, Run the VC++ utililty TRACER.EXE, "));
	pDC->TextOut(20,  40, CString("and make sure the Enable Tracing box is checked. "));
	pDC->TextOut(20,  60, CString(""));

#ifdef _DEBUG
	pDC->TextOut(20,  80, CString("This program has been compiled as a DEBUG version, and is ready "));
	pDC->TextOut(20, 100, CString("for use. "));
#else // _DEBUG
	pDC->TextOut(20,  80, CString("This program has been compiled as a RELEASE version. It must be"));
	pDC->TextOut(20, 100, CString("recompiled as a DEBUG version before trace output will be available."));
#endif // _DEBUG

	pDC->TextOut(20, 120, CString(""));
	pDC->TextOut(20, 140, CString("This program must be run from Visual Studio to see trace output. "));
	pDC->TextOut(20, 160, CString("Output can be seen in the DEBUG output window while the program "));
	pDC->TextOut(20, 180, CString("is running or after execution stops. "));
	pDC->TextOut(20, 200, CString(""));
	pDC->TextOut(20, 220, CString("To view the output, switch to the Visual Studio. View the output "));
	pDC->TextOut(20, 240, CString("window, and click on the DEBUG tab."));
	pDC->TextOut(20, 260, CString(""));
	pDC->TextOut(20, 280, CString("To generate some output, click a mouse button or double click the "));
	pDC->TextOut(20, 300, CString("left button."));
}

/////////////////////////////////////////////////////////////////////////////
// CTraceMacroView diagnostics

#ifdef _DEBUG
void CTraceMacroView::AssertValid() const
{
	CView::AssertValid();
}

void CTraceMacroView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CTraceMacroDoc* CTraceMacroView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTraceMacroDoc)));
	return (CTraceMacroDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// Important functions.

// --------------------------------------------------------------------------
void CTraceMacroView::DoSomethingImportant()
{
	IT_IT("CTraceMacroView::DoSomethingImportant()");

	m_Importance.IncImportance();

	IT_COMMENT1("This important function has been called %d times!", 
												m_Importance.GetImportance());
}

// --------------------------------------------------------------------------
void CTraceMacroView::DoSomethingVeryImportant()
{
	IT_IT("CTraceMacroView::DoSomethingVeryImportant()");

	DoSomethingImportant();

	m_pVeryImportance->RecordImportance( m_Importance.GetImportance() );

	DoSomethingImportant();
}

/////////////////////////////////////////////////////////////////////////////
// CTraceMacroView message handlers

// --------------------------------------------------------------------------
void CTraceMacroView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	IT_IT("CTraceMacroView::OnLButtonUp()");

	DoSomethingImportant();

	CView::OnLButtonUp(nFlags, point);
}


// --------------------------------------------------------------------------
void CTraceMacroView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	IT_IT("CTraceMacroView::OnLButtonDown()");

	CView::OnLButtonDown(nFlags, point);
}


// --------------------------------------------------------------------------
void CTraceMacroView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	IT_IT("CTraceMacroView::OnLButtonDblClk()");

	DoSomethingVeryImportant();

	CView::OnLButtonDblClk(nFlags, point);
}


// --------------------------------------------------------------------------
void CTraceMacroView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	
}


// --------------------------------------------------------------------------
void CTraceMacroView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	IT_IT("CTraceMacroView::OnRButtonDown()");

	CView::OnRButtonDown(nFlags, point);
}


// --------------------------------------------------------------------------
void CTraceMacroView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	IT_IT("CTraceMacroView::OnRButtonUp");

	// I/O error: Open non-existent file.
	::SetLastError( 0 );
	HANDLE h = ::CreateFile( "NoSuchFile.xyz", GENERIC_READ, 0, NULL, 
								OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
	if (h == INVALID_HANDLE_VALUE)
	{
		IT_TRACE_IO_ERR;
	}
	else
	{
		::CloseHandle( h );
	}

	CView::OnRButtonUp(nFlags, point);
}
